import { world, system } from "@minecraft/server";
import { ActionFormData, MessageFormData, ModalFormData } from "@minecraft/server-ui";

const itemArray = [
    "miraculous:bee_spinning_top",
    "miraculous:vesperia_spinning_top"
];
world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.itemComponentRegistry.registerCustomComponent("miraculous:bee_spinning_top_on_use", {
		onUse: ({ source, itemStack }) => {
			const player = source;
			if (player.isSneaking) return;
			const cooldown = itemStack.getComponent("cooldown").getCooldownTicksRemaining(player);
			if (cooldown === 47) {
				const viewDirection = player.getViewDirection();
				const speedFactor = 2;
				const scaledDirection = {
					x: viewDirection.x * speedFactor,
					y: viewDirection.y * speedFactor,
					z: viewDirection.z * speedFactor
				};
				if (itemArray.includes(itemStack.typeId)) {
					let projectileType = null;
					if (!player.hasTag("vesperia")) {
						projectileType = "miraculous:bee_spinning_top_projectile";
					} else if (player.hasTag("vesperia")) {
						projectileType = "miraculous:vesperia_spinning_top_projectile";
					}
					if (projectileType) {
						const projectile = player.dimension.spawnEntity(projectileType, {
							x: player.location.x + viewDirection.x,
							y: player.location.y + 1.5 + viewDirection.y,
							z: player.location.z + viewDirection.z
						});
						const projectileComponent = projectile.getComponent("projectile");
						projectileComponent.shoot(scaledDirection);
						projectile.triggerEvent("yoyo_seat");
					}
					player.playSound("yoyo", { pitch: 1.0, location: player.location, volume: 1.0 });
				}
			}
		}
	});
});
world.beforeEvents.itemUse.subscribe(event => {
	const itemType = event.itemStack.typeId;
	const player = event.source;
	if (event.source.isSneaking) {
		if (itemArray.includes(itemType) && !event.source.hasTag('use_venom')) {
			player.runCommandAsync('tellraw @a[tag=!use_venom] {\"rawtext\":[{\"text\":\"§g<\"},{\"selector\":\"@s\"},{\"text\":\">§r\"},{\"translate\":\"tile.miraculous:venom\"}]}');
			player.runCommandAsync('clear @s miraculous:venom');
			player.runCommandAsync('give @s[tag=!use_venom] miraculous:venom');
			player.runCommandAsync('tag @s[tag=!full_power] add use_venom');
			player.runCommandAsync('tag @s[tag=use_venom, tag=!full_power] add bee_miraculous_timer_on');
			player.runCommandAsync('playsound venom @a ~~~');
		}
	}
});