import { world, system } from "@minecraft/server";
import { ActionFormData, MessageFormData, ModalFormData } from "@minecraft/server-ui";

const itemArray = [
    "miraculous:bubble_sword",
    "miraculous:lady_wifi_cell_phone"
];
world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.itemComponentRegistry.registerCustomComponent("miraculous:transmission_item_on_use", {
		onUse: ({ source, itemStack }) => {
			const player = source;
			if (player.isSneaking) return;
			const cooldown = itemStack.getComponent("cooldown").getCooldownTicksRemaining(player);
			if (cooldown === 47) {
				const viewDirection = player.getViewDirection();
				const speedFactor = 2;
				const scaledDirection = {
					x: viewDirection.x * speedFactor,
					y: viewDirection.y * speedFactor,
					z: viewDirection.z * speedFactor
				};
				if (itemArray.includes(itemStack.typeId)) {
					let projectileType = null;
					if (player.hasTag("bubbler")) {
						projectileType = "miraculous:bubble";
					} else if (player.hasTag("lady_wifi")) {
						projectileType = "miraculous:pause_projectile";
					}
					if (projectileType) {
						const projectile = player.dimension.spawnEntity(projectileType, {
							x: player.location.x + viewDirection.x,
							y: player.location.y + 1.5 + viewDirection.y,
							z: player.location.z + viewDirection.z
						});
						const projectileComponent = projectile.getComponent("projectile");
						projectileComponent.shoot(scaledDirection);
					}
				}
			}
		}
	});
});
world.beforeEvents.itemUse.subscribe(event => {
	const itemType = event.itemStack.typeId;
		if (event.source.isSneaking) {
			if (itemType == 'miraculous:bubble_sword') {
				event.source.runCommandAsync('clear @s miraculous:bubble_sword');
				event.source.runCommandAsync('summon miraculous:butterfly ~~~~~ miraculous:magic_butterfly');
				event.source.runCommandAsync('tag @s[tag=is_transformed] remove is_transformed');
				event.source.runCommandAsync('replaceitem entity @s slot.armor.head 0 air');
				event.source.removeTag('bubbler');
			}
		}
		if (event.source.isSneaking) {
			if (itemType == 'miraculous:lady_wifi_cell_phone') {
				event.source.runCommandAsync('clear @s miraculous:lady_wifi_cell_phone');
				event.source.runCommandAsync('summon miraculous:butterfly ~~~~~ miraculous:magic_butterfly');
				event.source.runCommandAsync('tag @s[tag=is_transformed] remove is_transformed');
				event.source.runCommandAsync('replaceitem entity @s slot.armor.head 0 air');
				event.source.removeTag('lady_wifi');
			}
		}
		if (event.source.isSneaking) {
			if (itemType == 'miraculous:mr_pigeon_whistle') {
				event.source.runCommandAsync('clear @s miraculous:mr_pigeon_whistle');
				event.source.runCommandAsync('summon miraculous:butterfly ~~~~~ miraculous:magic_butterfly');
				event.source.runCommandAsync('tag @s[tag=is_transformed] remove is_transformed');
				event.source.runCommandAsync('replaceitem entity @s slot.armor.head 0 air');
				event.source.removeTag('mr_pigeon');
			}
		}
	}
);