import { world, system } from "@minecraft/server";
import { ActionFormData, MessageFormData, ModalFormData } from "@minecraft/server-ui";

const itemArray = [
    "miraculous:fox_flute",
    "miraculous:rena_furtive_flute"
];
world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.itemComponentRegistry.registerCustomComponent("miraculous:fox_flute_on_use", {
		onUse: ({ source, itemStack }) => {
			const player = source;
			if (player.isSneaking) return;
			const cooldown = itemStack.getComponent("cooldown").getCooldownTicksRemaining(player);
			if (cooldown === 47) {
				const viewDirection = player.getViewDirection();
				const speedFactor = 2;
				const scaledDirection = {
					x: viewDirection.x * speedFactor,
					y: viewDirection.y * speedFactor,
					z: viewDirection.z * speedFactor
				};
				if (itemArray.includes(itemStack.typeId)) {
					let projectileType = null;
					if (player.hasTag("mirage_creeper")) {
						projectileType = "miraculous:mirage_creeper_projectile";
						player.playSound("mirage_blow", { pitch: 1.0, location: player.location, volume: 1.0 });
						player.removeTag('mirage_creeper');
						player.runCommandAsync("tag @s[tag=full_power] remove use_mirage");

					} else if (player.hasTag("mirage_skeleton")) {
						projectileType = "miraculous:mirage_skeleton_projectile";
						player.playSound("mirage_blow", { pitch: 1.0, location: player.location, volume: 1.0 });
						player.removeTag('mirage_skeleton');
						player.runCommandAsync("tag @s[tag=full_power] remove use_mirage");

					} else if (player.hasTag("mirage_zombie")) {
						projectileType = "miraculous:mirage_zombie_projectile";
						player.playSound("mirage_blow", { pitch: 1.0, location: player.location, volume: 1.0 });
						player.removeTag('mirage_zombie');
						player.runCommandAsync("tag @s[tag=full_power] remove use_mirage");
					}
					if (projectileType) {
						const projectile = player.dimension.spawnEntity(projectileType, {
							x: player.location.x + viewDirection.x,
							y: player.location.y + 1.5 + viewDirection.y,
							z: player.location.z + viewDirection.z
						});
						const projectileComponent = projectile.getComponent("projectile");
						projectileComponent.shoot(scaledDirection);
					}
				}
			}
		}
	});
});
world.beforeEvents.itemUse.subscribe(event => {
	const itemType = event.itemStack.typeId;
	const player = event.source;
		if (itemArray.includes(itemType) && player.isSneaking == true ) {
			player.runCommandAsync('event entity @e[family=mirage] despawn');
		}
		if (itemArray.includes(itemType) && !player.hasTag('use_mirage') && player.isSneaking == false ) system.run(() => foxMiraculousPowerMenu(player));

	function foxMiraculousPowerMenu() {
		const form = new ActionFormData();
			form.title('title.choose_mirage');
			form.button('button.creeper');
			form.button('button.skeleton');
			form.button('button.zombie');
			form.show(player).then(r => {
				if (r.canceled) {
					return;
				}
				if (r.selection !== null) {
					player.runCommandAsync("event entity @e[family=mirage] despawn");
					player.runCommandAsync('playsound mirage @a ~~~');
					player.runCommandAsync("tag @s[tag=!use_mirage] add use_mirage");
					player.runCommandAsync("tag @s[tag=use_mirage, tag=!full_power] add fox_miraculous_timer_on");
					player.runCommandAsync('tag @s[tag=use_mirage, tag=!full_power] add timer_on');
					player.runCommandAsync('tellraw @a[tag=!rena_furtive] {\"rawtext\":[{\"text\":\"§6<\"},{\"selector\":\"@s\"},{\"text\":\">§r\"},{\"translate\":\"tile.miraculous:mirage\"}]}');
					player.runCommandAsync('tellraw @a[tag=rena_furtive] {\"rawtext\":[{\"text\":\"§b<\"},{\"selector\":\"@s\"},{\"text\":\">§r\"},{\"translate\":\"tile.miraculous:mirage\"}]}');
					player.removeTag('mirage_creeper');
					player.removeTag('mirage_skeleton');
					player.removeTag('mirage_zombie');
				}
				if(r.selection == 0) {
					player.addTag('mirage_creeper');
				}
				if(r.selection == 1) {
					player.addTag('mirage_skeleton');
				}
				if(r.selection == 2) {
					player.addTag('mirage_zombie');
				}
			}
		)
	}
});