import { world, system } from "@minecraft/server";
import { ActionFormData, MessageFormData, ModalFormData } from "@minecraft/server-ui";

const itemArray = [
    "miraculous:ladybug_yoyo",
    "miraculous:chronobug_yoyo",
    "miraculous:bugnoire_yoyo"
];
world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.itemComponentRegistry.registerCustomComponent("miraculous:ladybug_yoyo_on_use", {
		onUse: ({ source, itemStack }) => {
			const player = source;
			if (player.isSneaking) return;
			const cooldown = itemStack.getComponent("cooldown").getCooldownTicksRemaining(player);
			if (cooldown === 47) {
				const viewDirection = player.getViewDirection();
				const speedFactor = 2;
				const scaledDirection = {
					x: viewDirection.x * speedFactor,
					y: viewDirection.y * speedFactor,
					z: viewDirection.z * speedFactor
				};
				if (itemArray.includes(itemStack.typeId)) {
					let projectileType = null;
					if (!player.hasTag("chronobug") && !player.hasTag("is_black_cat")) {
						projectileType = "miraculous:ladybug_yoyo_projectile";
					} else if (player.hasTag("chronobug") && !player.hasTag("is_black_cat")) {
						projectileType = "miraculous:chronobug_yoyo_projectile";
					} else if (player.hasTag("is_black_cat") && player.hasTag("is_ladybug")) {
						projectileType = "miraculous:bugnoire_yoyo_projectile";
					}
					if (projectileType) {
						const projectile = player.dimension.spawnEntity(projectileType, {
							x: player.location.x + viewDirection.x,
							y: player.location.y + 1.5 + viewDirection.y,
							z: player.location.z + viewDirection.z
						});
						const projectileComponent = projectile.getComponent("projectile");
						projectileComponent.shoot(scaledDirection);
						projectile.triggerEvent("yoyo_seat");
					}
					player.playSound("yoyo", { pitch: 1.0, location: player.location, volume: 1.0 });
				}
			}
		}
	});
});
world.beforeEvents.itemUse.subscribe(event => {
	const itemType = event.itemStack.typeId;
	const player = event.source;
	if (event.source.isSneaking) {
		if (itemArray.includes(itemType) && event.source.hasTag('is_random')) {
			const commands = [
				'give @s miraculous:lucky_charm_sword 1 0 {\"keep_on_death\":{}}',
				'give @s miraculous:lucky_charm_axe 1 0 {\"keep_on_death\":{}}',
				'give @s miraculous:lucky_charm_pickaxe 1 0 {\"keep_on_death\":{}}',
				'give @s miraculous:lucky_charm_shovel 1 0 {\"keep_on_death\":{}}',
				'give @s miraculous:lucky_charm_hoe 1 0 {\"keep_on_death\":{}}',
				'give @s miraculous:lucky_charm_mace 1 0 {\"keep_on_death\":{}}',
				'give @s miraculous:lucky_charm_hammer 1 0 {\"keep_on_death\":{}}',
				'give @s miraculous:lucky_charm_flint_and_steel 1 0 {\"keep_on_death\":{}}',
				'summon miraculous:motorcycle_lucky_charm ~~10~~~',
				'summon miraculous:lamborghini_lucky_charm ~~10~~~',
				'summon miraculous:hovercraft_lucky_charm ~~10~~~',
				'summon miraculous:plane_lucky_charm ~~10~~~'
			];
			const randomCommand = commands[Math.floor(Math.random() * commands.length)];
			player.runCommandAsync("event entity @e[family=vehicles] despawn");
			player.runCommandAsync('clear @s miraculous:lucky_charm_sword 0 1');
			player.runCommandAsync('clear @s miraculous:lucky_charm_axe 0 1');
			player.runCommandAsync('clear @s miraculous:lucky_charm_pickaxe 0 1');
			player.runCommandAsync('clear @s miraculous:lucky_charm_shovel 0 1');
			player.runCommandAsync('clear @s miraculous:lucky_charm_hoe 0 1');
			player.runCommandAsync('clear @s miraculous:lucky_charm_mace 0 1');
			player.runCommandAsync('clear @s miraculous:lucky_charm_hammer 0 1');
			player.runCommandAsync('clear @s miraculous:lucky_charm_flint_and_steel 0 1');
			player.runCommandAsync(randomCommand);
			player.runCommandAsync("tag @s[tag=!use_lucky_charm, tag=!full_power] add use_lucky_charm");
			player.runCommandAsync("tag @s[tag=use_lucky_charm, tag=!full_power] add ladybug_miraculous_timer_on");
			player.runCommandAsync('playsound lucky_charm @a ~~~');
			player.runCommandAsync('event entity @s[tag=is_lucky_charm] lucky_charm_transformation');
			player.runCommandAsync('tag @s remove is_random');
		}
	}
});
world.beforeEvents.itemUse.subscribe(event => {
	const itemType = event.itemStack.typeId;
	const player = event.source;
		if (itemArray.includes(itemType) && !player.hasTag('is_random') && !player.hasTag('use_lucky_charm') && player.isSneaking == true ) system.run(() => ladybugYoyoMenu(player));

	function ladybugYoyoMenu() {
		const form = new ActionFormData();
			form.title('title.ladybug_yoyo_menu');
			form.button('button.lucky.charm.random');
			form.button('button.choose.lucky.charm');
			form.show(player).then(r => {
				if(r.selection == 0) {
					player.addTag('is_random');
				}
				if(r.selection == 1) {
					chooseLuckyCharm(player);
				}
			}
		)
	}
	function chooseLuckyCharm() {
		const form = new ActionFormData();
			form.title('title.choose_lucky_charm');
			form.button('item.miraculous:lucky_charm_sword', 'textures/items/miraculous/ladybug_miraculous/lucky_charm/lucky_charm_sword');
			form.button('item.miraculous:lucky_charm_axe', 'textures/items/miraculous/ladybug_miraculous/lucky_charm/lucky_charm_axe');
			form.button('item.miraculous:lucky_charm_pickaxe', 'textures/items/miraculous/ladybug_miraculous/lucky_charm/lucky_charm_pickaxe');
			form.button('item.miraculous:lucky_charm_shovel', 'textures/items/miraculous/ladybug_miraculous/lucky_charm/lucky_charm_shovel');
			form.button('item.miraculous:lucky_charm_hoe', 'textures/items/miraculous/ladybug_miraculous/lucky_charm/lucky_charm_hoe');
			form.button('item.miraculous:lucky_charm_mace', 'textures/items/miraculous/ladybug_miraculous/lucky_charm/lucky_charm_mace');
			form.button('item.miraculous:lucky_charm_hammer', 'textures/items/miraculous/ladybug_miraculous/lucky_charm/lucky_charm_hammer');
			form.button('item.miraculous:lucky_charm_flint_and_steel', 'textures/items/miraculous/ladybug_miraculous/lucky_charm/lucky_charm_flint_and_steel');
			form.button('button.motorcycle', 'textures/icons/motorcycle_lucky_charm');
			form.button('button.lamborghini', 'textures/icons/lamborghini_lucky_charm');
			form.button('button.hovercraft', 'textures/icons/hovercraft_lucky_charm');
			form.button('button.airplane', 'textures/icons/plane_lucky_charm');
			form.show(player).then(r => {
				if (r.canceled) {
					return;
				}
				if (r.selection !== null) {
					player.runCommandAsync("event entity @e[family=vehicles] despawn");
					player.runCommandAsync('clear @s miraculous:lucky_charm_sword 0 1');
					player.runCommandAsync('clear @s miraculous:lucky_charm_axe 0 1');
					player.runCommandAsync('clear @s miraculous:lucky_charm_pickaxe 0 1');
					player.runCommandAsync('clear @s miraculous:lucky_charm_shovel 0 1');
					player.runCommandAsync('clear @s miraculous:lucky_charm_hoe 0 1');
					player.runCommandAsync('clear @s miraculous:lucky_charm_mace 0 1');
					player.runCommandAsync('clear @s miraculous:lucky_charm_hammer 0 1');
					player.runCommandAsync('clear @s miraculous:lucky_charm_flint_and_steel 0 1');
					player.runCommandAsync("tag @s[tag=!use_lucky_charm, tag=!full_power] add use_lucky_charm");
					player.runCommandAsync("tag @s[tag=use_lucky_charm, tag=!full_power] add ladybug_miraculous_timer_on");
					player.runCommandAsync('playsound lucky_charm @a ~~~');
					player.runCommandAsync('event entity @s[tag=is_lucky_charm] lucky_charm_transformation');
				}
				if(r.selection == 0) {
					player.runCommandAsync("give @s miraculous:lucky_charm_sword 1 0 {\"keep_on_death\":{}}");
				}
				if(r.selection == 1) {
					player.runCommandAsync("give @s miraculous:lucky_charm_axe 1 0 {\"keep_on_death\":{}}");
				}
				if(r.selection == 2) {
					player.runCommandAsync("give @s miraculous:lucky_charm_pickaxe 1 0 {\"keep_on_death\":{}}");
				}
				if(r.selection == 3) {
					player.runCommandAsync("give @s miraculous:lucky_charm_shovel 1 0 {\"keep_on_death\":{}}");
				}
				if(r.selection == 4) {
					player.runCommandAsync("give @s miraculous:lucky_charm_hoe 1 0 {\"keep_on_death\":{}}");
				}
				if(r.selection == 5) {
					player.runCommandAsync("give @s miraculous:lucky_charm_mace 1 0 {\"keep_on_death\":{}}");
				}
				if(r.selection == 6) {
					player.runCommandAsync("give @s miraculous:lucky_charm_hammer 1 0 {\"keep_on_death\":{}}");
				}
				if(r.selection == 7) {
					player.runCommandAsync("give @s miraculous:lucky_charm_flint_and_steel 1 0 {\"keep_on_death\":{}}");
				}
				if(r.selection == 8) {
					player.runCommandAsync("summon miraculous:motorcycle_lucky_charm ~~10~~~");
				}
				if(r.selection == 9) {
					player.runCommandAsync("summon miraculous:lamborghini_lucky_charm ~~10~~~");
				}
				if(r.selection == 10) {
					player.runCommandAsync("summon miraculous:hovercraft_lucky_charm ~~10~~~");
				}
				if(r.selection == 11) {
					player.runCommandAsync("summon miraculous:plane_lucky_charm ~~10~~~");
				}
			}
		)
	}
});